
#include "gamemanager.h"
#include "map.h"

#include <algorithm>

GameManager::GameManager() {
	trackObject = 0;
}

void GameManager::render()
{
	// Work out the visible part of the map
	const int VIEW_DIST = 4;
	int cx = (int) (viewCentre.x / TileSize);
	int cy = (int) (viewCentre.y / TileSize);
	int minx = std::max(0, cx - VIEW_DIST);
	int miny = std::max(0, cy - VIEW_DIST);
	int maxx = std::max(0, cx + VIEW_DIST);
	int maxy = std::max(0, cy + VIEW_DIST);
	// render it.
	theMap.render(minx, miny, maxx, maxy);
	// Render objects.
	// Optimisation: culling.
	for (GameObjectIterator i = things.begin(); i != things.end(); i++) {
		GameObject *obj = *i;
		obj->render();
	}
}

static bool IsAlive(GameObject *obj)
{
	return ! obj->dead;
}

static bool IsDead(GameObject *obj)
{
	return obj->dead;
}

void GameManager::tick()
{
	cleanList(deadThings);
	// Take any new objects and stick them into things.
	for (GameObjectIterator i = newThings.begin(); i != newThings.end(); i++) {
		things.push_back(*i);
	}
	newThings.clear(); // clear out newThings
	
	// Tick them.
	for (GameObjectIterator i = things.begin(); i != things.end(); i++) {
		GameObject *obj = *i;
		obj->tick();
	}
	collideTest();
	
	if (trackObject) {
		if (trackObject->dead) {
			trackObject = 0;
		} else {
			viewCentre = trackObject->location + viewOffset;
			viewCentre.y += TileSize; // up a bit.
		}
	}
	// Clean out dead objects, move them into a dead list.
	std::remove_copy_if(things.begin(), things.end(), back_inserter(deadThings), IsAlive);
	things.erase(std::remove_if(things.begin(), things.end(), IsDead), things.end());
	
}

void GameManager::collideTest()
{
	// We will test everything against trackObject.
	if (trackObject == 0) {
		return;
	}
	for (GameObjectIterator i = things.begin(); i != things.end(); i++) {
		GameObject *obj = *i;
		if (obj == trackObject) continue;
		if (obj->collidable) {
			double xdist = std::fabs(obj->location.x - trackObject->location.x);
			if (xdist < (obj->radius + trackObject->radius)) {
				// Check Y collision.
				bool collided = false;
				if (obj->location.y >= trackObject->location.y) {
					if (obj->location.y < (trackObject->location.y + trackObject->height)) {
						collided = true;
					}
				} else {
					if (trackObject->location.y < (obj->location.y + obj->height)) {
						collided = true;
					}
				}
				if (collided) {
					handleCollision(trackObject, obj);
				}
			}
		}
	}
}

void GameManager::handleCollision(GameObject *obj1, GameObject * obj2)
{
	obj1->collided(obj2);
	obj2->collided(obj1);
}

void GameManager::cleanList( GameObjectList &list)
{
	for (GameObjectIterator i = list.begin(); i != list.end(); i++) {
		delete(*i);
	}
	list.clear();
}

void GameManager::triggerSound(ResourceManager::SampleType sample, const Vect3d &where)
{
	SAMPLE *s = resourceManager.getSample(sample);
	play_sample(s, 192 /* vol */, 128 /* pan */, 1000, 0);
}


