#ifndef GAMEMANAGER_INCLUDED
 #define GAMEMANAGER_INCLUDED

#include "vector3d.h"
#include "resourcemanager.h"
#include <vector>

class GameManager;

class GameObject {
	protected:
		GameManager *manager;
	public:
		bool dead;
		bool collidable;
		bool deadly;
		Vect3d location;
		Vect3d velocity;
		float radius;
		float height;
		void setManager(GameManager *m) {
			manager = m;
		}
		
		virtual void render() { }
		virtual void tick() { }
		virtual void collided(GameObject * other) { }
		
		virtual ~GameObject() { }
		
		GameObject() : dead(false), collidable(false), deadly(false) { }
};

class GameManager {
	private:
		typedef std::vector <GameObject *> GameObjectList;
		typedef GameObjectList::iterator GameObjectIterator;
		GameObjectList things;
		GameObjectList deadThings;
		GameObjectList newThings;
		
		void cleanList( GameObjectList &list);
		void handleCollision(GameObject *obj1, GameObject * obj2);
		void collideTest();
	public:
		GameObject *trackObject;
		Vect3d viewCentre;
		Vect3d viewOffset;
		
		GameManager();
		~GameManager() {
			cleanList(things);
			cleanList(newThings);
			cleanList(deadThings);
		}
		void render();
		void tick();
		
		void add(GameObject *thing) {
			thing->setManager(this);
			newThings.push_back(thing);
		}
		
		void setTrack(GameObject *what) {
			trackObject = what;
		}
		
		/* Used between levels to get rid of the previous
	 	 * Level's stuff.
		 */
		void cleanUp() 
		{
			trackObject = 0;
			cleanList(things);
			cleanList(newThings);
			cleanList(deadThings);
		}
		
		void triggerSound(ResourceManager::SampleType sample, const Vect3d &where);
};

#endif // GAMEMANAGER_INCLUDED

