#include "resourcemanager.h"
#include <alleggl.h>
#include <stdexcept>

ResourceManager ::ResourceManager()
{
	textureIds.resize(MaxTexture + 1);
}

static int loadFromFile(const std::string & filename, bool alpha = false)
{
	PALETTE pal;
	BITMAP *bmp = load_bitmap(filename.c_str(), pal);
	if (bmp == 0) {
		throw std::runtime_error("Failed to load texture: " + filename);
	}
	int textureId = allegro_gl_make_texture_ex(AGL_TEXTURE_FLIP  /* flags */,
		bmp,
		alpha ? GL_RGBA : GL_RGB); // internal_format	
	destroy_bitmap(bmp);
	if (textureId == 0) {
		throw std::runtime_error("Failed to convert texture: " + filename);
	}
	return textureId;	
}

static SAMPLE * loadSample(const std::string & filename)
{
	SAMPLE *s = load_sample(filename.c_str());
	if (s == 0) {
		throw std::runtime_error("Failed to load sample: " + filename);
	}
	return s;
}

void ResourceManager::loadAll()
{
	// Don't convert these as they may contain alpha.
	set_color_conversion(COLORCONV_NONE);
	textureIds[PlayerSkin] = loadFromFile("gfx/jacktex.tga");
	textureIds[PlayerDeadSkin] = loadFromFile("gfx/dead.tga");
	textureIds[BrickTexture] = loadFromFile("gfx/bricks.tga");
	textureIds[EarthTexture] = loadFromFile("gfx/ground.tga");
	textureIds[StoneTexture] = loadFromFile("gfx/stone.tga");
	textureIds[GenericTexture] = loadFromFile("gfx/generic.tga");
	textureIds[StalkTexture] = loadFromFile("gfx/stalktexture.tga");
	textureIds[BackgroundTexture] = loadFromFile("gfx/bg.tga");
	textureIds[TitleTexture] = loadFromFile("gfx/title.tga");
	textureIds[BeanTexture] = loadFromFile("gfx/bean.tga", true);
	textureIds[TreasureTexture] = loadFromFile("gfx/treasure.tga", true);
	textureIds[GiantTexture] = loadFromFile("gfx/gianttexture.tga");
	
	models.resize(MaxModel);
	models[Stalk1].load("models/stalk1.model");
	models[Stalk2].load("models/stalk2.model");
	models[Stalk3].load("models/stalk3.model");
	models[Jack].load("models/jack.model");
	models[JackWalking1].load("models/jackwalk1.model");
	models[JackWalking2].load("models/jackwalk2.model");
	models[JackWalking3].load("models/jackwalk3.model");
	models[JackWalking4].load("models/jackwalk4.model");
	models[JackClimbing1].load("models/jackclimb1.model");
	models[JackClimbing2].load("models/jackclimb2.model");
	models[JackDead].load("models/jackdead.model");
	models[Spikes].load("models/spikes.model");
	models[Door1].load("models/door1.model");
	models[Door2].load("models/door2.model");
	models[Giant].load("models/giant.model");
	
	samples.resize(MaxSample);
	samples[Step1] = loadSample("sounds/step1.wav");
	samples[Step2] = loadSample("sounds/step2.wav");
	samples[Stomp] = loadSample("sounds/stomp.wav");
	samples[Growing] = loadSample("sounds/swanee.wav");
	samples[Throw] = loadSample("sounds/throw.wav");
	samples[Death] = loadSample("sounds/aargh.wav");

}

ResourceManager resourceManager;

